BidirPartialPattern

Struct BidirPartialPattern 

pub struct BidirPartialPattern<T, U, V, R> { /* private fields */ }
SendChannel & BidirChannel partial Join Pattern.

Implementations

Source

impl<T, U, V, R> BidirPartialPattern<T, U, V, R>
where T: Any + Send, U: Any + Send, V: Any + Send, R: Any + Send,

pub fn then_do<F>(self, f: F)
where F: Fn(T, U, V) -> R + Send + Clone + 'static,

Create full JoinPattern and send request to add it to Junction.

Create a full Join Pattern by taking the channels that are part of the partial pattern and adding a function to be executed when there is at least one message sent on each channel. Attempt to add the Join Pattern to the Junction after creation.

Panics

Panics if it was not possible to send the request to add the newly create Join Pattern to the Junction.