pub struct BidirPartialPattern<T, U, V, R> { /* private fields */ }Expand description
SendChannel & BidirChannel partial Join Pattern.
Implementations
Sourceimpl<T, U, V, R> BidirPartialPattern<T, U, V, R>
impl<T, U, V, R> BidirPartialPattern<T, U, V, R>
Sourcepub fn then_do<F>(self, f: F)
pub fn then_do<F>(self, f: F)
Create full JoinPattern and send request to add it to Junction.
Create a full Join Pattern by taking the channels that are part of
the partial pattern and adding a function to be executed when there
is at least one message sent on each channel. Attempt to add the
Join Pattern to the Junction after creation.
Panics
Panics if it was not possible to send the request to add the newly
create Join Pattern to the Junction.